% Script for generating the PO-PI curve for a given process gp
% The controller used is an ideal PI controller
% Written by: Martin S. Foss, fall 2012

%clc
clear all

global gp msEq iaeWeights manWeights iaeTuning

%Adding "sharedFiles" to MatLab search directory
curDir = pwd;
mainDir = fileparts(curDir);
sharedDir = fullfile(mainDir,'sharedFiles');
addpath(sharedDir);

%%
tic
modelId = 9; 
modelName = {'case1','case2','case3','case4','case5','case6',...
             'case7','case8','case9','case10'};
gp = model(modelId);    % getting the model

%% Finding the iaeWeights for the PI-controller
    fprintf('Finding Optimal IAE Weights (PI-controller) \n')
    fprintf('Case: %g\n',modelId)
    fprintf('***************************\n')
    fprintf('\n')
    fprintf('costFun(iae) \t minTuning \t\t Ms \t exitFlag \n')
    fprintf('-------------------------------------------------\n')

msEq = 1.59;   %Ms for the iaeWeights 
opt = optimset('algorithm','active-set','Display','off','TolCon',1e-4); 
               %'active-set', 'trust-region-reflective', 'interior-point',
               %'interior-point-convex', 'levenberg-marquardt', 
               %'trust-region-dogleg', 'lm-line-search', or 'sqp'.

%Initial solution guesses
X0 = [0.5 0.4 0   %case 1
      0.5 0.3 0   %case 2
      0.5 0.4 0   %case 3
      1.1 0.7 0   %case 4
      0.8 0.5 0   %case 5
      1.5 1.2 0   %case 6
      1.3 0.9 0   %case 7
      1.0 0.6 0   %case 8
      1.9 1.4 0]; %case 9
x0 = X0(modelId,:)';

iaeWeights = [1; 1];    %cost function weigths
manWeights = [1 0];

Aeq = [0 0 1];          %constraints
Beq = 0;

[minTuningSp,iaeSp,exitFlagSp] =  fmincon(@costFun,x0,[],[],Aeq,Beq,...
                                          [0;0;0],[],@conFun,opt);

    fprintf('%0.2f \t \t %.2f %.2f %.2f \t %.2f\t %i \t \n',iaeSp,...
            minTuningSp, msEq, exitFlagSp)
    
%%
%Initial solution guesses
X0 = [0.5 0.4 0   %case 1
      0.5 0.3 0   %case 2
      0.5 0.4 0   %case 3
      0.9 0.8 0   %case 4
      0.7 0.5 0   %case 5
      1.3 1.3 0   %case 6
      1.1 0.9 0   %case 7
      0.9 0.6 0   %case 8
      1.5 1.6 0]; %case 9
x0 = X0(modelId,:)';
    
manWeights = [0 1];

[minTuningD,iaeD,exitFlagD] = fmincon(@costFun,x0,[],[],Aeq,Beq,...
                                      [0;0;0],[],@conFun,opt);

    fprintf('%0.2f \t \t %.2f %.2f %.2f \t %.2f\t %i \t \n',iaeD,...
            minTuningD, msEq, exitFlagD)
    fprintf('\n');
    fprintf('\n');
    fprintf('\n');
    fprintf('\n');

%% Generating curve
try 
    load(fullfile(mainDir,'dataFiles',['resPOpid_',...
                  num2str(modelName{modelId}),'.mat']))
catch me
    ME = MException(me.identifier,...
           'could not open file, check corrrect modelId and data folder!');
    throw(ME)
end
minSp = res.POpid.minSp.iae;
minD = res.POpid.minD.iae;
iaeWeights = [minSp; minD];

i = 1;            %iteration counter for command window printout
minTuning = [];   %matrices for storing results
costTuning = [];
iaeOptTun = [];

msSpace = 1.25:0.05:3; %the Ms search range

%Initial solution guesses
X0 = [0.2 0.2 0   %case 1
      0.3 0.2 0   %case 2
      0.2 0.2 0   %case 3
      0.4 0.4 0   %case 4
      0.5 0.4 0   %case 5
      0.6 0.6 0   %case 6
      0.5 0.4 0   %case 7
      0.4 0.3 0   %case 8
      1.2 1.1 0]; %case 9 
x0 = X0(modelId,:)';

manWeights = [.5, .5];

    fprintf('Generating the PO Curve\n')
    fprintf('***********************\n')
    fprintf('\n')
    fprintf('Number of iterations: %i \n', length(msSpace))
    fprintf('\n')
    fprintf('costFun(%.1f, %.1f) \t minTuning \t \t Ms',manWeights)
    fprintf(' \t exitFlag \t iterations left \n')
    fprintf('------------------------------------------------------------')
    fprintf('--------------------\n')
    
opt = optimset('algorithm','active-set','Display','off','TolCon',1e-4); 
               %'active-set', 'trust-region-reflective', 'interior-point', 
               %'interior-point-convex', 'levenberg-marquardt', 
               %'trust-region-dogleg', 'lm-line-search', or 'sqp'.

%Optimizing
for msEq = msSpace;

    [minTuningTemp, iaeTuningTemp, exitFlagTuningTemp] = ...
            fmincon(@costFun,x0,[],[],Aeq,Beq,[0;0;0],[],@conFun,opt);

    minTuning(:,i) = minTuningTemp;         %storing resutlts
    costTuning(i) = iaeTuningTemp;
    exitFlagTuning(i) = exitFlagTuningTemp;
    iaeOptTun(i,:) = iaeTuning;

    if modelId == 1 && i == 22 || modelId == 1 && i == 32 || ...
       modelId == 3 && i == 22 || modelId == 9 && i == 29 || ...
       modelId == 9 && i >= 32
        x0 = 0.9*minTuningTemp;
    elseif modelId == 5 && i == 1 || modelId == 5 && i == 2
        x0 = 1.2*minTuningTemp;
    else
        x0 = minTuningTemp;
    end

    fprintf('%0.2f \t \t \t %.2f  %.2f %.2f \t %.2f\t %i \t \t\t %i \n',...
            iaeTuningTemp, minTuningTemp, msEq, exitFlagTuningTemp,...
            length(msSpace)-i)

    i = i + 1;  %updating iteration counter
end

    fprintf('\n')
    fprintf('Calculation Finished!\n')
    fprintf('=====================\n')
toc

%% Plotting the results
% Cost function, J, vs. Ms
figure(modelId)
clf
h(1) = plot(msSpace,costTuning);
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',14)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',14)
titleName = {'Case 1','Case 2','Case 3','Case 4','Case 5',...
             'Case 6','Case 7','Case 8','Case 9',};

title(titleName{modelId},'interpreter','latex','FontSize',14)

%% Storing results (in the same struct as the PO (PID) tunings)
res.POpi.minSp.tuning = minTuningSp;
res.POpi.minSp.iae = iaeSp;
res.POpi.minSp.exitFlag = exitFlagSp;

res.POpi.minD.tuning = minTuningD;
res.POpi.minD.iae = iaeD;
res.POpi.minD.exitFlag = exitFlagD;

res.POpi.minTuning.ms = msSpace;
res.POpi.minTuning.tuning = minTuning;
res.POpi.minTuning.costFun = costTuning;
res.POpi.minTuning.exitFlag = exitFlagTuning;
res.POpi.minTuning.iae = iaeOptTun;

pause
%saving struct
save([mainDir,'\dataFiles\','resPO_',...
      num2str(modelName{modelId}),'.mat'],'res') 
save(['poPIresults\','resPO_',...
      num2str(modelName{modelId}),'.mat'],'res')
  
%saving figures
saveas(h(1),['figures\','POpiCurve_',num2str(modelName{modelId}),'.fig'])

restoredefaultpath