% Script for generating the PO-PID curve for a given process gp
% The controller used is an ideal PID controller
% Written by: Martin S. Foss, fall 2012

%clc
clear all

global gp msEq iaeWeights manWeights iaeTuning

%Adding "sharedFiles" to MatLab search directory
curDir = pwd;
mainDir = fileparts(curDir);
sharedDir = fullfile(mainDir,'sharedFiles');
addpath(sharedDir);

%%
tic
modelId = 9;
gp = model(modelId);    %getting the model

%% Finding the iaeWeights 
    fprintf('Finding Optimal IAE Weights \n')
    fprintf('Case: %g\n',modelId)
    fprintf('***************************\n')
    fprintf('\n')
    fprintf('costFun(iae) \t minTuning \t\t Ms \t exitFlag \n')
    fprintf('-------------------------------------------------\n')

msEq = 1.59;   %Ms for the iaeWeights 
opt = optimset('algorithm','active-set','Display','off','TolCon',1e-4); 
               %'active-set', 'trust-region-reflective', 'interior-point',
               %'interior-point-convex', 'levenberg-marquardt', 
               %'trust-region-dogleg', 'lm-line-search', or 'sqp'.

%Initial solution guesses
X0 = [0.9 0.5 0.4   %case 1
      1.0 0.6 0.6   %case 2
      0.8 0.7 0.3   %case 3
      2.2 1.5 0.8   %case 4
      1.7 0.9 0.8   %case 5
      3.2 2.4 0.8   %case 6
      2.9 1.9 1.0   %case 7
      2.2 1.2 1.0   %case 8
      4.2 3.2 1.0]; %case 9 
x0 = X0(modelId,:)';

iaeWeights = [1; 1];    %cost function weigths
manWeights = [1 0];

[minTuningSp,iaeSp,exitFlagSp] = fmincon(@costFun,x0,[],[],[],[],...
                                         [0;0;0],[],@conFun,opt);

    fprintf('%0.2f \t \t %.2f %.2f %.2f \t %.2f\t %i \t \n',iaeSp,...
            minTuningSp, msEq, exitFlagSp)
    
%%
%Initial solution guesses
X0 = [0.9 0.7 0.5   %case 1
      1.0 0.6 0.6   %case 2
      0.8 0.7 0.35  %case 3
      2.2 2.3 0.8   %case 4
      1.7 1.3 0.8   %case 5
      3.3 5.0 0.8   %case 6
      3.0 3.4 1.0   %case 7
      2.2 1.8 1.0   %case 8
      4.5 7.6 1.0]; %case 9 
x0 = X0(modelId,:)';
    
manWeights = [0 1];

[minTuningD,iaeD,exitFlagD] = fmincon(@costFun,x0,[],[],[],[],....
                                      [0;0;0],[],@conFun,opt);

    fprintf('%0.2f \t \t %.2f %.2f %.2f \t %.2f\t %i \t \n',iaeD,...
            minTuningD, msEq, exitFlagD)
    fprintf('\n');
    fprintf('\n');
    fprintf('\n');
    fprintf('\n');

iaeWeights = [iaeSp; iaeD];

%% Generating curve
i = 1;              %iteration counter for command window printout
minTuning = [];     %matrices for storing results
costTuning = [];
iaeOptTun = [];

msSpace = 1.25:0.05:3;  %Ms search range

%Initial solution guesses
X0 = [0.7 0.5 0.4   %case 1
      0.8 0.6 0.5   %case 2
      0.4 0.4 0.3   %case 3
      1.1 1.1 0.4   %case 4
      0.9 0.7 0.5   %case 5
      1.6 1.9 0.4   %case 6
      1.5 1.4 0.5   %case 7
      1.1 0.9 0.6   %case 8
      2.0 3.0 0.6]; %case 9 
x0 = X0(modelId,:)';

manWeights = [.5, .5];

    fprintf('Generating the PO Curve\n')
    fprintf('***********************\n')
    fprintf('\n')
    fprintf('Number of iterations: %i \n', length(msSpace))
    fprintf('\n')
    fprintf('costFun(%.1f, %.1f) \t minTuning \t \t Ms ',manWeights)
    fprintf('\t exitFlag \t iterations left \n')
    fprintf('------------------------------------------------------------')
    fprintf('--------------------\n')
    
opt = optimset('algorithm','active-set','Display','off','TolCon',1e-4); 
               %'active-set', 'trust-region-reflective', 'interior-point',
               %'interior-point-convex', 'levenberg-marquardt', 
               %'trust-region-dogleg', 'lm-line-search', or 'sqp'.

%Optimizing
for msEq = msSpace;

    [minTuningTemp, iaeTuningTemp, exitFlagTuningTemp] = ...
                   fmincon(@costFun,x0,[],[],[],[],[0;0;0],[],@conFun,opt);

    minTuning(:,i) = minTuningTemp;         %storing resutlts
    costTuning(i) = iaeTuningTemp;
    exitFlagTuning(i) = exitFlagTuningTemp;
    iaeOptTun(i,:) = iaeTuning;

    if modelId == 2 && i == 1 || modelId == 3 && i == 1 ||...
       modelId == 5 && i == 1 
        x0 = 1.1*minTuningTemp;
    elseif modelId == 6 && i == 7 || modelId == 9 && i == 32 
        x0 = 0.9*minTuningTemp;
    elseif modelId == 9 && i == 17
        x0 = 0.9*minTuningTemp;
    elseif modelId == 9 && i == 16 || modelId == 9 && i >= 34
        x0 = 0.91*minTuningTemp;
    else
        x0 = minTuningTemp;
    end

    fprintf('%0.2f \t \t \t %.2f  %.2f %.2f \t %.2f\t %i \t \t\t %i \n',...
            iaeTuningTemp, minTuningTemp, msEq, exitFlagTuningTemp,...
            length(msSpace)-i)

    i = i + 1; %updating iteration counter
end

    fprintf('\n')
    fprintf('Calculation Finished!\n')
    fprintf('=====================\n')
toc

%% Ploting the results
%Cost function, J vs. Ms
figure(modelId)
clf
h(1) = plot(msSpace,costTuning);
axis([1.2 3 0.8 1.3])
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',14)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',14)
titleName = {'Case 1','Case 2','Case 3','Case 4','Case 5',...
             'Case 6','Case 7','Case 8','Case 9'};

title(titleName{modelId},'interpreter','latex','FontSize',14)

%% Storing results
modelName = {'case1','case2','case3','case4','case5','case6',...
             'case7','case8','case9'};
         
res.case = num2str(modelName{modelId});

res.POpid.minSp.tuning = minTuningSp;
res.POpid.minSp.iae = iaeSp;
res.POpid.minSp.exitFlag = exitFlagSp;

res.POpid.minD.tuning = minTuningD;
res.POpid.minD.iae = iaeD;
res.POpid.minD.exitFlag = exitFlagD;

res.POpid.minTuning.ms = msSpace;
res.POpid.minTuning.tuning = minTuning;
res.POpid.minTuning.costFun = costTuning;
res.POpid.minTuning.exitFlag = exitFlagTuning;
res.POpid.minTuning.iae = iaeOptTun;

% pause
% saving struct
save([mainDir,'\dataFiles\','resPOpid_',...
      num2str(modelName{modelId}),'.mat'],'res')    %saving "globaly"
save(['poPIDresults\','resPOpid_',...
      num2str(modelName{modelId}),'.mat'],'res')	%saving "localy"

% saving figures
saveas(h(1),['figures\','POpidCurve_',num2str(modelName{modelId}),'.fig'])

restoredefaultpath