% Script calculating tuning parameters (parallel form) for PO(PID), 
% PO(PI), SIMC-PID and SIMC-PI
% Written by: Martin S. Foss, fall 2012

% clc
clear all

%Adding "sharedFiles" to MatLab search directory
curDir = pwd;
mainDir = fileparts(curDir);
sharedDir = fullfile(mainDir,'sharedFiles');
addpath(sharedDir);

%%
% for m = 1:9
modelId = 9; %m

    fprintf('Finding tuningparamaters (parallel form) \n')
    fprintf('Case: %g\n',modelId)
    fprintf('***************************\n')
    fprintf('\n')
    
msSet = 1.7;        %define Ms value

modelName = {'case1','case2','case3',...
             'case4','case5','case6',...
             'case7','case8','case9'};
     
load(fullfile(mainDir,'dataFiles',['resSimc_',...
                num2str(modelName{modelId}),'.mat'])) %loading datafiles

%Collecting PO and SIMC tunings
index = find(res.POpid.minTuning.ms == msSet);
tuning{1} = res.POpid.minTuning.tuning(:,index(1));

index2 = find(res.POpi.minTuning.ms == msSet);
tuning{2} = res.POpi.minTuning.tuning(:,index2(1));

index3 = find(res.simcPID.simcMs <= msSet);
tuning{3} = res.simcPID.simcTuning(:,index3(1));

index4 = find(res.simcPI.simcMs <= msSet);
tuning{4} = res.simcPI.simcTuning(:,index4(1));

x = [1 2 3 4];

tuningParallel = [];         %setting up result matrix
for i=1:length(tuning)
    Kc = tuning{i}(1);       %finding Kc
    taui = Kc/tuning{i}(2);  %calculating tau_i
    
    if i == 2 || i == 4      %calculating tau_d
        taud = NaN;             
    else
        taud = tuning{i}(3)/Kc;
    end
    
    tuningParallel(:,i) = [Kc; taui; taud];     %storing results
end

res.tuningPara = tuningParallel;

%% Saving results
save([mainDir,'\dataFiles\','resTunPar_',num2str(modelName{modelId}),...
        '.mat'],'res');
save(['tuningResults\','resTunPar_',num2str(modelName{modelId}),...
        '.mat'],'res');
% end

restoredefaultpath