% function for maximum time delay error tuning as discribed in:
%
% Ruscio, D. D. (2010). On Tuning PI Controllers for Integrating Plus Time Delay Systems. 
%       Modeling, Identification and Control: A Norwegian Research Bulletin, 31(4), 145-164. 
%       doi:10.4173/mic.2010.4.3
%
% For integrating with delay prosesses: 1/s*exp(-theta*s)
%
% input: 1st order model + tuning parameter
% returns: mdet tuning paramer for a PI controller on the form gc = K + I/s

function [tuning, gc] = mdet(gp,c,rdelay)

% determining the model

t = gp.den{1}(1);
d = totaldelay(gp);
g = gp.num{1}(end);
z = zero(gp);
p = pole(gp);

if nargin == 1
    tc = d;
end

if length(gp.den{1}) > 2
    error('model order to high!')
elseif isempty(z) == 0
    error('model cannot contain zeros!')
end


if p == 0 % integrating process
    k = g/t;
    f = 1 +sqrt(1+4/c^2)/2;
    a = atan(sqrt(f)*c)/sqrt(f);
    beta = (c/a)*(rdelay+1);
    alfa = a/(rdelay+1);
    K = alfa/(k*d);
    I = K/(beta*d);    
elseif isempty(p)==1; % pure delay process
   error('works only for lag with time delay process!')
else
   error('works only for lag with time delay process!')
end
   

tuning = [K;I];
gc = controller(K,I);

return