% function for simc PID tuning
% input: 2nd order model + tuning parameter
% returns: simc tuning paramer for a PID controller on the form 
% gc = K + I/s + D*s
% Written by: Martin S. Foss, fall 2012

function [tuning, gc] = simcPID(gp,tc)

%Determining the model
d = totaldelay(gp);         %time delay
g = gp.num{1}(end);         %gain
z = zero(gp);               %zeros
p = pole(gp);               %poles
t1 = abs(p(2)); 	        %time constant 1 (largest)
t2 = abs(p(2)/p(1));        %time constant 2 (smallest)

if nargin == 1
    tc = d;
end

if length(gp.den{1}) > 3
    disp('model order to high')
    return
elseif isempty(z) == 0
    disp('model cannot contain zeros')
    return
end

%SIMC-tuning (series)
Kc = 1/g*(t1/(tc+d));            
taui = min(t1,4*(tc+d));
taud = t2;

%SIMC-tuning (parallel)
K_merk = Kc*(1+taud/taui);         
I_merk = taui*(1+taud/taui);       
D_merk = taud/(1+taud/taui);

%SIMC-tuning (given parameterization)
K = K_merk;
I = K_merk/I_merk;
D = K_merk*D_merk;

tuning = [K I D]';
gc = controller(K,I,D);
return